package com.ruoyi.warehouseBusiness.excel;

import java.math.BigDecimal;

/**
 * 账单明细分项实体
 */
public class BillDetailsSubItem {
    // 业务类型
    private String fBilltype;

    // 作业类型
    private String twlfBusinesstype;

    // 客户名称
    private Long fCorpid;

    // 提单号
    private String fStatementno;

    // 货名
    private String tgfName;

    // 客户
    private String tcfName;

    // 件数
    private BigDecimal fQty;

    // 重量
    private BigDecimal fNetweight;

    // 体积
    private BigDecimal fVolumn;

    // 入库时间
    private String fOriginalbilldate;

    // 出库时间
    private String fBsdate;

    // 计费天数
    private int fBillingdays;

    // 金额
    private BigDecimal fAmount;

    // 计划员
    private String createBy;

    // 计费时间
    private String fChargedate;

    // 结费时间
    private String fStlamountdate;

    // 仓库
    private String fhName;

    // 备注
    private String remark;

    // 统计范围
    private String dateRange;

    public String getDateRange() {
        return dateRange;
    }

    public void setDateRange(String dateRange) {
        this.dateRange = dateRange;
    }

    public String getfBilltype() {
        return fBilltype;
    }

    public void setfBilltype(String fBilltype) {
        this.fBilltype = fBilltype;
    }

    public String getTwlfBusinesstype() {
        return twlfBusinesstype;
    }

    public void setTwlfBusinesstype(String twlfBusinesstype) {
        this.twlfBusinesstype = twlfBusinesstype;
    }

    public Long getfCorpid() {
        return fCorpid;
    }

    public void setfCorpid(Long fCorpid) {
        this.fCorpid = fCorpid;
    }

    public String getfStatementno() {
        return fStatementno;
    }

    public void setfStatementno(String fStatementno) {
        this.fStatementno = fStatementno;
    }

    public String getTgfName() {
        return tgfName;
    }

    public void setTgfName(String tgfName) {
        this.tgfName = tgfName;
    }

    public String getTcfName() {
        return tcfName;
    }

    public void setTcfName(String tcfName) {
        this.tcfName = tcfName;
    }

    public BigDecimal getfQty() {
        return fQty;
    }

    public void setfQty(BigDecimal fQty) {
        this.fQty = fQty;
    }

    public BigDecimal getfNetweight() {
        return fNetweight;
    }

    public void setfNetweight(BigDecimal fNetweight) {
        this.fNetweight = fNetweight;
    }

    public BigDecimal getfVolumn() {
        return fVolumn;
    }

    public void setfVolumn(BigDecimal fVolumn) {
        this.fVolumn = fVolumn;
    }

    public String getfOriginalbilldate() {
        return fOriginalbilldate;
    }

    public void setfOriginalbilldate(String fOriginalbilldate) {
        this.fOriginalbilldate = fOriginalbilldate;
    }

    public String getfBsdate() {
        return fBsdate;
    }

    public void setfBsdate(String fBsdate) {
        this.fBsdate = fBsdate;
    }

    public int getfBillingdays() {
        return fBillingdays;
    }

    public void setfBillingdays(int fBillingdays) {
        this.fBillingdays = fBillingdays;
    }

    public BigDecimal getfAmount() {
        return fAmount;
    }

    public void setfAmount(BigDecimal fAmount) {
        this.fAmount = fAmount;
    }

    public String getCreateBy() {
        return createBy;
    }

    public void setCreateBy(String createBy) {
        this.createBy = createBy;
    }

    public String getfChargedate() {
        return fChargedate;
    }

    public void setfChargedate(String fChargedate) {
        this.fChargedate = fChargedate;
    }

    public String getfStlamountdate() {
        return fStlamountdate;
    }

    public void setfStlamountdate(String fStlamountdate) {
        this.fStlamountdate = fStlamountdate;
    }

    public String getFhName() {
        return fhName;
    }

    public void setFhName(String fhName) {
        this.fhName = fhName;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }
}
